@@ -1,24 +1,24 @@
 #include "common.h"
 
-int do_proc_sys_kernel_random_entropy_avail(int update_every, unsigned long long dt) {
+int do_proc_sys_kernel_random_entropy_avail(int update_every, usec_t dt) {
+    (void)dt;
+
     static procfile *ff = NULL;
 
-    if(dt) {} ;
-
-    if(!ff) {
+    if(unlikely(!ff)) {
         char filename[FILENAME_MAX + 1];
         snprintfz(filename, FILENAME_MAX, "%s%s", global_host_prefix, "/proc/sys/kernel/random/entropy_avail");
         ff = procfile_open(config_get("plugin:proc:/proc/sys/kernel/random/entropy_avail", "filename to monitor", filename), "", PROCFILE_FLAG_DEFAULT);
+        if(unlikely(!ff)) return 1;
     }
-    if(!ff) return 1;
 
     ff = procfile_readall(ff);
-    if(!ff) return 0; // we return 0, so that we will retry to open it next time
+    if(unlikely(!ff)) return 0; // we return 0, so that we will retry to open it next time
 
-    unsigned long long entropy = strtoull(procfile_lineword(ff, 0, 0), NULL, 10);
+    unsigned long long entropy = str2ull(procfile_lineword(ff, 0, 0));
 
     RRDSET *st = rrdset_find_bytype("system", "entropy");
-    if(!st) {
+    if(unlikely(!st)) {
         st = rrdset_create("system", "entropy", NULL, "entropy", NULL, "Available Entropy", "entropy", 1000, update_every, RRDSET_TYPE_LINE);
         rrddim_add(st, "entropy", NULL, 1, 1, RRDDIM_ABSOLUTE);
     }
